var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import path from 'path';
import fs from 'fs';
import os from 'os';
import util from 'util';
const fsExistsAsync = util.promisify(fs.exists);
const fsReadFileAsync = util.promisify(fs.readFile);
function getDefaultSearchPlaces(name) {
    return [
        'package.json',
        `.${name}rc.json`,
        `.${name}rc.js`,
        `${name}.config.js`,
    ];
}
function getSearchPaths(startDir, stopDir) {
    return startDir
        .split(path.sep)
        .reduceRight((acc, _, ind, arr) => {
        const currentPath = arr.slice(0, ind + 1).join(path.sep);
        if (!acc.passedStopDir)
            acc.searchPlaces.push(currentPath);
        if (currentPath === stopDir)
            acc.passedStopDir = true;
        return acc;
    }, { searchPlaces: [], passedStopDir: false }).searchPlaces;
}
function getOptions(name, options) {
    return Object.assign({
        stopDir: os.homedir(),
        searchPlaces: getDefaultSearchPlaces(name),
        ignoreEmptySearchPlaces: true,
        transform: (result) => result,
    }, (options !== null && options !== void 0 ? options : {}));
}
function getSearchItems(searchPlaces, searchPaths) {
    return searchPaths.reduce((acc, searchPath) => {
        searchPlaces.forEach(fileName => {
            acc.push({
                fileName,
                filePath: path.join(searchPath, fileName),
            });
        });
        return acc;
    }, []);
}
export function lilconfig(name, options = {}) {
    const { ignoreEmptySearchPlaces, searchPlaces, stopDir, transform, } = getOptions(name, options);
    return {
        search(searchFrom = process.cwd()) {
            return __awaiter(this, void 0, void 0, function* () {
                const searchPaths = getSearchPaths(searchFrom, stopDir);
                const result = {
                    config: null,
                    path: '',
                };
                const searchItems = getSearchItems(searchPlaces, searchPaths);
                for (const { fileName, filePath } of searchItems) {
                    const exists = yield fsExistsAsync(filePath);
                    if (!exists)
                        continue;
                    // handle package.json
                    if (fileName === 'package.json') {
                        try {
                            const pkg = require(filePath);
                            if (name in pkg) {
                                result.config = pkg[name];
                                result.path = filePath;
                                break;
                            }
                        }
                        catch (err) { }
                        continue;
                    }
                    // handle other type of configs
                    const content = String(yield fsReadFileAsync(filePath));
                    const isEmpty = content.trim() === '';
                    if (isEmpty && ignoreEmptySearchPlaces)
                        continue;
                    try {
                        result.config = require(filePath);
                        result.path = filePath;
                        if (isEmpty)
                            result.isEmpty = isEmpty;
                    }
                    catch (err) {
                        result.error = err;
                        result.config = null;
                    }
                    break;
                }
                // not found
                if (result.path === '' && result.config === null)
                    return transform(null);
                return transform(result);
            });
        },
        load(filePath) {
            return __awaiter(this, void 0, void 0, function* () {
                const exists = yield fsExistsAsync(filePath);
                if (!exists)
                    return null;
                const { base } = path.parse(filePath);
                if (base === 'package.json') {
                    const pkg = yield import(filePath);
                    return transform(name in pkg
                        ? {
                            config: pkg[name],
                            path: filePath,
                        }
                        : null);
                }
                const result = {
                    config: null,
                    path: filePath,
                };
                // handle other type of configs
                const content = String(yield fsReadFileAsync(filePath));
                const isEmpty = content.trim() === '';
                if (isEmpty && ignoreEmptySearchPlaces)
                    return transform(null);
                // cosmiconfig returns undefined for empty files
                result.config = isEmpty ? undefined : yield import(filePath);
                return transform(isEmpty ? Object.assign(Object.assign({}, result), { isEmpty, config: undefined }) : result);
            });
        },
    };
}
export function lilconfigSync(name, options = {}) {
    const { ignoreEmptySearchPlaces, searchPlaces, stopDir, transform, } = getOptions(name, options);
    return {
        search(searchFrom = process.cwd()) {
            const searchPaths = getSearchPaths(searchFrom, stopDir);
            const result = {
                config: null,
                path: '',
            };
            const searchItems = getSearchItems(searchPlaces, searchPaths);
            for (const { fileName, filePath } of searchItems) {
                const exists = fs.existsSync(filePath);
                if (!exists)
                    continue;
                // handle package.json
                if (fileName === 'package.json') {
                    try {
                        const pkg = require(filePath);
                        if (name in pkg) {
                            result.config = pkg[name];
                            result.path = filePath;
                            break;
                        }
                    }
                    catch (err) { }
                    continue;
                }
                // handle other type of configs
                const content = String(fs.readFileSync(filePath));
                const isEmpty = content.trim() === '';
                if (isEmpty && ignoreEmptySearchPlaces)
                    continue;
                try {
                    result.config = require(filePath);
                    result.path = filePath;
                    if (isEmpty)
                        result.isEmpty = isEmpty;
                }
                catch (err) {
                    result.error = err;
                    result.config = null;
                }
                break;
            }
            // not found
            if (result.path === '' && result.config === null)
                return transform(null);
            return transform(result);
        },
        load(filePath) {
            if (!fs.existsSync(filePath))
                return null;
            const { base } = path.parse(filePath);
            if (base === 'package.json') {
                const pkg = require(filePath);
                return transform(name in pkg
                    ? {
                        config: pkg[name],
                        path: filePath,
                    }
                    : null);
            }
            const result = {
                config: null,
                path: filePath,
            };
            // handle other type of configs
            const content = fs.readFileSync(filePath).toString();
            const isEmpty = content.trim() === '';
            if (isEmpty && ignoreEmptySearchPlaces)
                return transform(null);
            // cosmiconfig returns undefined for empty files
            result.config = isEmpty ? undefined : require(filePath);
            return transform(isEmpty ? Object.assign(Object.assign({}, result), { isEmpty, config: undefined }) : result);
        },
    };
}
